/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.area;

import java.util.List;
import java.util.concurrent.Callable;
import java.util.logging.Level;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.Field;
import nl.cloudfarming.client.model.SensorData;
import nl.cloudfarming.client.model.SensorRun;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.util.Cancellable;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Task for linking sensor data to a specific field
 * 
 * @author Timon Veenstra
 */
public class LinkSensorFieldTask implements Callable<LinkSensorFieldTaskResult> {

    private static final AppLogger LOG = AppLogFactory.getLogger(LinkSensorFieldTask.class);
    private final Field field;
    private static final int BUFFER_SIZE = 500;
    private Cancellable cancellable;
    private long linkedSensorDataCount = 0L;
    private long newSensorRunCount = 0L;

    LinkSensorFieldTask(Field field) {
        this.field = field;
    }

    //FIXME move linking to fieldplan module
    @Override
    public LinkSensorFieldTaskResult call() throws Exception {
        throw new UnsupportedOperationException("FIXME move linking to fieldplan module");
//        try {
//            Geometry plotGeometry = field.getGeometry();
//
//            // Find all sensordata within the bounds of the maximum dimensions of the plot
//            List<SensorData> unlinkedSensorData = getService().findUnlinkedSensorDataInBox(plotGeometry.getEnvelope());
//
//            // Start Progressbar
//            InternalProgressHandle progressHandle = createProgressHandle();
//            progressHandle.start(unlinkedSensorData.size());
//
//            List<SensorRun> sensorRuns = new ArrayList<SensorRun>();
//
//            if (unlinkedSensorData.size() > 0) {
//                List<SensorData> sensorDataBuffer = new ArrayList<SensorData>();
//                for (SensorData sensorData : unlinkedSensorData) {
//                    Geometry point = GeometryUtil.getPoint(sensorData.getLongitude(), sensorData.getLatitude());
//
//                    //
//                    // check intersection with real plot geometry
//                    //
//                    if (plotGeometry.intersects(point)) {
//                        // Get sensorrun for this field and the DataSensorFile()
//                        SensorRun sensorRun = getSensorRun(field, sensorData.getSensorDataFile());
//
//                        // Keep track of sensorruns being processed by this linking task to calculate sensor averages after linking.
//                        if (!sensorRuns.contains(sensorRun)) {
//                            sensorRuns.add(sensorRun);
//                        }
//                        sensorData.setSensorRun(sensorRun);
//                        sensorDataBuffer.add(sensorData);
//                        if (sensorDataBuffer.size() % BUFFER_SIZE == 0) {
//                            logFine("persisting sensor data cache");
//                            saveBuffer(sensorDataBuffer);
//                        }
//                    }
//                    progressHandle.progress();
//                }
//                if (!sensorDataBuffer.isEmpty()) {
//                    logFine("persisting remaining sensor data cache");
//                    saveBuffer(sensorDataBuffer);
//                }
//                logFine("link process finished for field {0}", field.getName());
//                // Clear list
//                unlinkedSensorData.clear();
//            }
//            progressHandle.finish();
//
//            // Calculate averages for each sensorrun
//            // In theory there can be more then one sensorrun present for a field.
//            calculateAverages(sensorRuns);
//
//        } catch (Exception e) {
//            LOG.error(e.toString());
//            throw new Exception(e);
//        }
//        return null;
    }

    /**
     * Persist sensor data cache
     * 
     * @param sensorDataBuffer
     * @throws Exception 
     */
    private void saveBuffer(List<SensorData> sensorDataBuffer) throws Exception {
        throw new UnsupportedOperationException("FIXME move linking to fieldplan module");
//        service.persist(sensorDataBuffer);
//        linkedSensorDataCount += sensorDataBuffer.size();
//        sensorDataBuffer.clear();
    }
    
    /**
     * not using <tt>call</tt> return value since we want to be able to show 
     * counts for partially executed (i.e. cancelled) tasks.
     * 
     * @return 
     */
    public LinkSensorFieldTaskResult getResult() {
        return new LinkSensorFieldTaskResult(linkedSensorDataCount, newSensorRunCount);
    }

    /**
     * Calculate NDV/WDVI averages for specified sensorruns.
     * 
     * Method is protected to be able to unittest.
     * 
     * @param sensorRuns List of sensorruns to calculate averages for.
     */
    //FIXME:
    protected void calculateAverages(List<SensorRun> sensorRuns) {
        // Loop through specified sensorruns
        for (SensorRun sr : sensorRuns) {
            double ndvi = 0.0;
            double wdvi = 0.0;

            // Find sensordata belonging to current sensorrun
//            List<SensorData> data = service.findDataForRun(sr);
//
//            // Loop through data belonging to this run
//            for (SensorData sd : data) {
//                if (sd.getNdvi() != null) {
//                    ndvi = ndvi + sd.getNdvi().doubleValue();
//                }
//                if (sd.getWdvi() != null) {
//                    wdvi = wdvi + sd.getWdvi().doubleValue();
//                }
//            }
//
//            // Set averages for sensorrun
//            if (ndvi != 0) {
//                sr.setAverageNdvi(ndvi / data.size());
//            }
//            if (wdvi != 0) {
//                sr.setAverageWdvi(wdvi / data.size());
//            }
////            service.updateSensorRun(sr);
        }
    }

    public void setCancellable(Cancellable cancellable) {
        this.cancellable = cancellable;
    }


    InternalProgressHandle createProgressHandle() {
        return new InternalProgressHandle();
    }

    private void logFine(String logLine, Object... params) {
        if (LOG.getLevel() == Level.FINE) {
            LOG.log(Level.FINE, logLine, params);
        }
    }

    /**
     * internal progress handle to make interception possible
     */
    public class InternalProgressHandle {

        private ProgressHandle intern;
        private int total;
        private int progress = 0;

        public InternalProgressHandle() {
            intern = ProgressHandleFactory.createHandle(NbBundle.getMessage(LinkSensorTask.class, "link.process.plot.title", field.getName()), cancellable);
        }

        public void start(int workunits) {
            total = workunits;
            intern.start(workunits);
        }

        public void progress() {
            if (progress != total) {
                intern.progress(++progress);
                logFine("incrementing total progress to {0}", progress);
            }
        }

        public void finish() {
            intern.finish();
        }
    }
}
